import pandas as pd
import numpy as np
import scipy.stats as ss
import matplotlib.pyplot as plt
import japanize_matplotlib
import seaborn as sb
import stan
import arviz as az

"""
           Excelファイルからの読み込み
"""
in_file_name = input('入力データファイル(*.xlsx) = ')
xlsx = pd.ExcelFile(in_file_name)
data_xlsx = pd.read_excel(xlsx)

"""
            出力ファイルの設定
"""
f = open('Results.txt', 'w')
f.write('\n入力データファイル = ' + in_file_name)

keys = data_xlsx.keys()         #     １行目の変数名の読み込み
print('Keys...\n', keys)
case_name = keys[0]

y_name = keys[1]
x_names = keys[2:]
print(y_name)
print(x_names)
Data = data_xlsx.values
print(Data)
case_id = Data[:,0]
y = Data[:,1]
X = Data[:,2:]

print(case_id)
print(y)
print(X)
y_mean = np.mean(y)
y_sd = np.std(y)
f.write(f'\n\ny_mean = {y_mean:.3f},   y_sd = {y_sd:.3f}\n')
X = np.array(X, dtype='float')
x_means = np.mean(X, axis=0)
print(x_means)
x_sds = np.std(X, axis=0)
print(x_sds)
for j in range(len(x_names)):
    f.write(f'\nb{j+1}({x_names[j]}):   ')
    f.write(f'mean = {x_means[j]:.3f},   sd = {x_sds[j]:.3f}\n')

y = ss.zscore(np.array(y, dtype ='float'))
X = ss.zscore(np.array(X, dtype = 'float'))
print(y)
print(X)

N, M = np.shape(X)
print(N, M)

with open('MLReg.stan', 'r') as f_stan:
    sm = stan.build(f_stan.read(), data = {'N':N, 'M':M, 'y':y, 'X':X})
fit = sm.sample()

d_frm = fit.to_frame()
i_data = az.from_pystan(posterior = fit, posterior_model = sm)

print(az.summary(i_data))
f.write('\n\n')
f.write(az.summary(i_data).__str__())

az.plot_trace(i_data)
plt.show()

def med_mad_sd(x):  
    """   中央値とmad sdの計算
        Gelman et al. (2021), p.73
    """
    med = np.median(x)
    mad = np.median(np.abs(x - med))
    mad_sd = 1.483 * mad
    return med, mad_sd

b0_med, b0_mad_sd = med_mad_sd(d_frm['b0'])

print(f'\nb0_med = {b0_med:.3f},    b0_mad_sd = {b0_mad_sd:.3f}')
f.write(f'\n\nb0_med = {b0_med:.3f},    b0_mad_sd = {b0_mad_sd:.3f}\n')
sb.kdeplot(d_frm['b0'])
plt.title('Posterior distribution for b0' +
          f'\nMed. = {b0_med:.3f},   mad sd = {b0_mad_sd:.3f}')
plt.show()

b_med = np.empty(M)
b_mad_sd = np.empty(M)
for j in range(M):
    b_med[j], b_mad_sd[j] = med_mad_sd(d_frm[f'b.{j+1}'])
    print(x_names[j])
    print(b_med[j], b_mad_sd[j])
    f.write(f'\nb{j+1}({x_names[j]}):' +
            f'   med = {b_med[j]:.3f},  mad_sd = {b_mad_sd[j]:.3f}\n')
    sb.kdeplot(d_frm[f'b.{j+1}'])
    plt.title(f'Posterir distribution for b{j+1}: ' + x_names[j] +
              f'\nMed. = {b_med[j]:.3f},   mad sd = {b_mad_sd[j]:.3f}')
    plt.show()

y_hat = np.empty(N)
for i in range(N):
    y_hat[i] = b0_med + np.dot(b_med, X[i])

for i in range(N):
    plt.plot(y_hat[i], y[i], 'o', c='b')
    plt.text(y_hat[i], y[i], case_id[i])
plt.xlabel('$\hat{y}$(model)')
plt.ylabel('$y$(data)')
x_min = np.min(y_hat)
x_max = np.max(y_hat)
plt.plot([x_min, x_max], [x_min, x_max], label ='y(data) = $\hat{y}$(model)')
plt.legend()
plt.show()

print('Results.txt was saved.')
